@@ -10,7 +10,8 @@ use super::*;
 /// Note: this will likely cause the underlying stream to buffer as
 /// much data as you read.  Thus, it should only be used for peeking
 /// at the underlying `BufferedReader`.
-pub struct Dup<T: BufferedReader<C>, C> {
+#[derive(Debug)]
+pub struct Dup<T: BufferedReader<C>, C: fmt::Debug + Sync + Send> {
     reader: T,
 
     // The number of bytes that have been consumed.
@@ -20,18 +21,14 @@ pub struct Dup<T: BufferedReader<C>, C> {
     cookie: C,
 }
 
-impl<T: BufferedReader<C>, C> fmt::Display for Dup<T, C> {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "Dup ({} bytes consumed)",
-               self.cursor)
-    }
-}
+assert_send_and_sync!(Dup<T, C>
+                      where T: BufferedReader<C>,
+                            C: fmt::Debug);
 
-impl<T: BufferedReader<C>, C> fmt::Debug for Dup<T, C> {
+impl<T: BufferedReader<C>, C: fmt::Debug + Sync + Send> fmt::Display for Dup<T, C> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("Dup")
             .field("cursor", &self.cursor)
-            .field("reader", &self.reader)
             .finish()
     }
 }
@@ -45,16 +42,16 @@ impl<T: BufferedReader<()>> Dup<T, ()> {
     }
 }
 
-impl<T: BufferedReader<C>, C> Dup<T, C> {
+impl<T: BufferedReader<C>, C: fmt::Debug + Sync + Send> Dup<T, C> {
     /// Like `new()`, but uses a cookie.
     ///
     /// The cookie can be retrieved using the `cookie_ref` and
     /// `cookie_mut` methods, and set using the `cookie_set` method.
     pub fn with_cookie(reader: T, cookie: C) -> Self {
         Dup {
-            reader: reader,
+            reader,
             cursor: 0,
-            cookie: cookie,
+            cookie,
         }
     }
 
@@ -69,7 +66,7 @@ impl<T: BufferedReader<C>, C> Dup<T, C> {
     }
 }
 
-impl<T: BufferedReader<C>, C> io::Read for Dup<T, C> {
+impl<T: BufferedReader<C>, C: fmt::Debug + Sync + Send> io::Read for Dup<T, C> {
     fn read(&mut self, buf: &mut [u8]) -> Result<usize, io::Error> {
         let data = self.reader.data(self.cursor + buf.len())?;
         assert!(data.len() >= self.cursor);
@@ -84,7 +81,7 @@ impl<T: BufferedReader<C>, C> io::Read for Dup<T, C> {
     }
 }
 
-impl<T: BufferedReader<C>, C> BufferedReader<C> for Dup<T, C> {
+impl<T: BufferedReader<C>, C: fmt::Debug + Send + Sync> BufferedReader<C> for Dup<T, C> {
     fn buffer(&self) -> &[u8] {
         let data = self.reader.buffer();
         assert!(data.len() >= self.cursor);
