@@ -7,29 +7,38 @@ use super::*;
 
 /// Wraps a `Read`er.
 ///
-/// This is useful when reading from a file, and it even works with a
-/// `&[u8]` (but `Memory` is more efficient).
-pub struct Generic<T: io::Read, C> {
+/// This is useful when reading from a generic `std::io::Read`er.  To
+/// read from a file, use [`File`].  To read from a buffer, use
+/// [`Memory`].  Both are more efficient than `Generic`.
+///
+///   [`File`]: struct.File.html
+///   [`Memory`]: struct.Memory.html
+pub struct Generic<T: io::Read + Send + Sync, C: fmt::Debug + Sync + Send> {
     buffer: Option<Box<[u8]>>,
     // The next byte to read in the buffer.
     cursor: usize,
     // The preferred chunk size.  This is just a hint.
     preferred_chunk_size: usize,
-    // XXX: This is pub for the decompressors.  It would be better to
-    // change this to some accessor method.
-    pub reader: Box<T>,
+    // The wrapped reader.
+    reader: T,
+    // Stashed error, if any.
+    error: Option<Error>,
 
     // The user settable cookie.
     cookie: C,
 }
 
-impl<T: io::Read, C> fmt::Display for Generic<T, C> {
+assert_send_and_sync!(Generic<T, C>
+                      where T: io::Read,
+                            C: fmt::Debug);
+
+impl<T: io::Read + Send + Sync, C: fmt::Debug + Sync + Send> fmt::Display for Generic<T, C> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "Generic")
     }
 }
 
-impl<T: io::Read, C> fmt::Debug for Generic<T, C> {
+impl<T: io::Read + Send + Sync, C: fmt::Debug + Sync + Send> fmt::Debug for Generic<T, C> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         let buffered_data = if let Some(ref buffer) = self.buffer {
             buffer.len() - self.cursor
@@ -44,7 +53,7 @@ impl<T: io::Read, C> fmt::Debug for Generic<T, C> {
     }
 }
 
-impl<T: io::Read> Generic<T, ()> {
+impl<T: io::Read + Send + Sync> Generic<T, ()> {
     /// Instantiate a new generic reader.  `reader` is the source to
     /// wrap.  `preferred_chuck_size` is the preferred chuck size.  If
     /// None, then the default will be used, which is usually what you
@@ -54,7 +63,7 @@ impl<T: io::Read> Generic<T, ()> {
     }
 }
 
-impl<T: io::Read, C> Generic<T, C> {
+impl<T: io::Read + Send + Sync, C: fmt::Debug + Sync + Send> Generic<T, C> {
     /// Like `new()`, but sets a cookie, which can be retrieved using
     /// the `cookie_ref` and `cookie_mut` methods, and set using
     /// the `cookie_set` method.
@@ -67,11 +76,27 @@ impl<T: io::Read, C> Generic<T, C> {
             preferred_chunk_size:
                 if let Some(s) = preferred_chunk_size { s }
                 else { DEFAULT_BUF_SIZE },
-            reader: Box::new(reader),
-            cookie: cookie,
+            reader,
+            error: None,
+            cookie,
         }
     }
 
+    /// Returns a reference to the wrapped writer.
+    pub fn reader_ref(&self) -> &T {
+        &self.reader
+    }
+
+    /// Returns a mutable reference to the wrapped writer.
+    pub fn reader_mut(&mut self) -> &mut T {
+        &mut self.reader
+    }
+
+    /// Returns the wrapped writer.
+    pub fn into_reader(self) -> T {
+        self.reader
+    }
+
     /// Return the buffer.  Ensure that it contains at least `amount`
     /// bytes.
     fn data_helper(&mut self, amount: usize, hard: bool, and_consume: bool)
@@ -84,6 +109,11 @@ impl<T: io::Read, C> Generic<T, C> {
         //          else { None });
 
 
+        // See if there is an error from the last invocation.
+        if let Some(e) = self.error.take() {
+            return Err(e);
+        }
+
         if let Some(ref buffer) = self.buffer {
             // We have a buffer.  Make sure `cursor` is sane.
             assert!(self.cursor <= buffer.len());
@@ -92,8 +122,6 @@ impl<T: io::Read, C> Generic<T, C> {
             assert_eq!(self.cursor, 0);
         }
 
-        let mut error = None;
-
         let amount_buffered
             = self.buffer.as_ref().map(|b| b.len() - self.cursor).unwrap_or(0);
         if amount > amount_buffered {
@@ -123,7 +151,7 @@ impl<T: io::Read, C> Generic<T, C> {
                     Err(err) => {
                         // Don't return yet, because we may have
                         // actually read something.
-                        error = Some(err);
+                        self.error = Some(err);
                         break;
                     },
                 }
@@ -149,14 +177,14 @@ impl<T: io::Read, C> Generic<T, C> {
         let amount_buffered
             = self.buffer.as_ref().map(|b| b.len() - self.cursor).unwrap_or(0);
 
-        if let Some(error) = error {
+        if self.error.is_some() {
             // An error occurred.  If we have enough data to fulfill
             // the caller's request, then don't return the error.
             if hard && amount > amount_buffered {
-                return Err(error);
+                return Err(self.error.take().unwrap());
             }
             if !hard && amount_buffered == 0 {
-                return Err(error);
+                return Err(self.error.take().unwrap());
             }
         }
 
@@ -178,13 +206,13 @@ impl<T: io::Read, C> Generic<T, C> {
     }
 }
 
-impl<T: io::Read, C> io::Read for Generic<T, C> {
+impl<T: io::Read + Send + Sync, C: fmt::Debug + Sync + Send> io::Read for Generic<T, C> {
     fn read(&mut self, buf: &mut [u8]) -> Result<usize, io::Error> {
         return buffered_reader_generic_read_impl(self, buf);
     }
 }
 
-impl<T: io::Read, C> BufferedReader<C> for Generic<T, C> {
+impl<T: io::Read + Send + Sync, C: fmt::Debug + Sync + Send> BufferedReader<C> for Generic<T, C> {
     fn buffer(&self) -> &[u8] {
         if let Some(ref buffer) = self.buffer {
             &buffer[self.cursor..]
