@@ -223,16 +223,16 @@
 //! [`BufReader`]: https://doc.rust-lang.org/stable/std/io/struct.BufReader.html
 //! [HTTP's chunk transfer encoding]: https://en.wikipedia.org/wiki/Chunked_transfer_encoding
 
-#[cfg(feature = "compression-deflate")]
-extern crate flate2;
-#[cfg(feature = "compression-bzip2")]
-extern crate bzip2;
-extern crate libc;
+#![warn(missing_docs)]
 
 use std::io;
 use std::io::{Error, ErrorKind};
 use std::cmp;
 use std::fmt;
+use std::convert::TryInto;
+
+#[macro_use]
+mod macros;
 
 mod generic;
 mod memory;
@@ -240,6 +240,7 @@ mod limitor;
 mod reserve;
 mod dup;
 mod eof;
+mod adapter;
 #[cfg(feature = "compression-deflate")]
 mod decompress_deflate;
 #[cfg(feature = "compression-bzip2")]
@@ -251,6 +252,7 @@ pub use self::limitor::Limitor;
 pub use self::reserve::Reserve;
 pub use self::dup::Dup;
 pub use self::eof::EOF;
+pub use self::adapter::Adapter;
 #[cfg(feature = "compression-deflate")]
 pub use self::decompress_deflate::Deflate;
 #[cfg(feature = "compression-deflate")]
@@ -258,6 +260,9 @@ pub use self::decompress_deflate::Zlib;
 #[cfg(feature = "compression-bzip2")]
 pub use self::decompress_bzip2::Bzip;
 
+// Common error type for file operations.
+mod file_error;
+
 // These are the different File implementations.  We
 // include the modules unconditionally, so that we catch bitrot early.
 #[allow(dead_code)]
@@ -295,7 +300,9 @@ fn vec_truncate(v: &mut Vec<u8>, len: usize) {
 }
 
 /// The generic `BufferReader` interface.
-pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display {
+pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
+  where C: fmt::Debug + Send + Sync
+{
     /// Returns a reference to the internal buffer.
     ///
     /// Note: this returns the same data as `self.data(0)`, but it
@@ -603,15 +610,16 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display {
     /// in big endian format.
     fn read_be_u16(&mut self) -> Result<u16, std::io::Error> {
         let input = self.data_consume_hard(2)?;
-        return Ok(((input[0] as u16) << 8) + (input[1] as u16));
+        // input holds at least 2 bytes, so this cannot fail.
+        Ok(u16::from_be_bytes(input[..2].try_into().unwrap()))
     }
 
     /// A convenience function for reading a 32-bit unsigned integer
     /// in big endian format.
     fn read_be_u32(&mut self) -> Result<u32, std::io::Error> {
         let input = self.data_consume_hard(4)?;
-        return Ok(((input[0] as u32) << 24) + ((input[1] as u32) << 16)
-                  + ((input[2] as u32) << 8) + (input[3] as u32));
+        // input holds at least 4 bytes, so this cannot fail.
+        Ok(u32::from_be_bytes(input[..4].try_into().unwrap()))
     }
 
     /// Reads until either `terminal` is encountered or EOF.
@@ -799,25 +807,37 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display {
     fn drop_eof(&mut self) -> Result<bool, std::io::Error> {
         let mut at_least_one_byte = false;
         loop {
-            match self.data_consume(DEFAULT_BUF_SIZE) {
-                Ok(ref buffer) => {
-                    if buffer.len() > 0 {
-                        at_least_one_byte = true;
-                    }
-
-                    if buffer.len() < DEFAULT_BUF_SIZE {
-                        // EOF.
-                        break;
-                    }
-                }
-                Err(err) =>
-                    return Err(err),
+            let n = self.data(DEFAULT_BUF_SIZE)?.len();
+            at_least_one_byte |= n > 0;
+            self.consume(n);
+            if n < DEFAULT_BUF_SIZE {
+                // EOF.
+                break;
             }
         }
 
         Ok(at_least_one_byte)
     }
 
+    /// A helpful debugging aid to pretty print a Buffered Reader stack.
+    ///
+    /// Uses the Buffered Readers' `fmt::Display` implementations.
+    fn dump(&self, sink: &mut dyn std::io::Write) -> std::io::Result<()>
+        where Self: std::marker::Sized
+    {
+        let mut i = 1;
+        let mut reader: Option<&dyn BufferedReader<C>> = Some(self);
+        while let Some(r) = reader {
+            {
+                let cookie = r.cookie_ref();
+                writeln!(sink, "  {}. {}, {:?}", i, r, cookie)?;
+            }
+            reader = r.get_ref();
+            i = i + 1;
+        }
+        Ok(())
+    }
+
     /// Boxes the reader.
     fn as_boxed<'a>(self) -> Box<dyn BufferedReader<C> + 'a>
         where Self: 'a + Sized
@@ -886,9 +906,9 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display {
 /// impl <T: BufferedReader> std::io::Read for T { ... }
 /// ```
 ///
-/// but, alas, Rust doesn't like that ("error[E0119]: conflicting
+/// but, alas, Rust doesn't like that ("error\[E0119\]: conflicting
 /// implementations of trait `std::io::Read` for type `&mut _`").
-pub fn buffered_reader_generic_read_impl<T: BufferedReader<C>, C>
+pub fn buffered_reader_generic_read_impl<T: BufferedReader<C>, C: fmt::Debug + Sync + Send>
         (bio: &mut T, buf: &mut [u8]) -> Result<usize, io::Error> {
     match bio.data_consume(buf.len()) {
         Ok(inner) => {
@@ -901,7 +921,7 @@ pub fn buffered_reader_generic_read_impl<T: BufferedReader<C>, C>
 }
 
 /// Make a `Box<BufferedReader>` look like a BufferedReader.
-impl <'a, C> BufferedReader<C> for Box<dyn BufferedReader<C> + 'a> {
+impl <'a, C: fmt::Debug + Sync + Send> BufferedReader<C> for Box<dyn BufferedReader<C> + 'a> {
     fn buffer(&self) -> &[u8] {
         return self.as_ref().buffer();
     }
@@ -999,7 +1019,7 @@ impl <'a, C> BufferedReader<C> for Box<dyn BufferedReader<C> + 'a> {
 //
 //   for i in $(seq 0 9999); do printf "%04d\n" $i; done > buffered-reader-test.txt
 #[cfg(test)]
-fn buffered_reader_test_data_check<'a, T: BufferedReader<C> + 'a, C>(bio: &mut T) {
+fn buffered_reader_test_data_check<'a, T: BufferedReader<C> + 'a, C: fmt::Debug + Sync + Send>(bio: &mut T) {
     use std::str;
 
     for i in 0 .. 10000 {
@@ -1053,7 +1073,7 @@ mod test {
     }
 
     #[cfg(test)]
-    fn buffered_reader_read_test_aux<'a, T: BufferedReader<C> + 'a, C>
+    fn buffered_reader_read_test_aux<'a, T: BufferedReader<C> + 'a, C: fmt::Debug + Sync + Send>
         (mut bio: T, data: &[u8]) {
         let mut buffer = [0; 99];
 
