@@ -9,24 +9,22 @@ use super::*;
 /// Note: because the `Reserve` doesn't generally know
 /// how much data can be read from the underlying `BufferedReader`,
 /// it causes at least N bytes to by buffered.
-pub struct Reserve<T: BufferedReader<C>, C> {
+#[derive(Debug)]
+pub struct Reserve<T: BufferedReader<C>, C: fmt::Debug + Sync + Send> {
     reader: T,
     reserve: usize,
 
     cookie: C,
 }
 
-impl<T: BufferedReader<C>, C> fmt::Display for Reserve<T, C> {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "Reserve ({} bytes)", self.reserve)
-    }
-}
+assert_send_and_sync!(Reserve<T, C>
+                      where T: BufferedReader<C>,
+                            C: fmt::Debug);
 
-impl<T: BufferedReader<C>, C> fmt::Debug for Reserve<T, C> {
+impl<T: BufferedReader<C>, C: fmt::Debug + Sync + Send> fmt::Display for Reserve<T, C> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("Reserve")
             .field("reserve", &self.reserve)
-            .field("reader", &self.reader)
             .finish()
     }
 }
@@ -41,7 +39,7 @@ impl<T: BufferedReader<()>> Reserve<T, ()> {
     }
 }
 
-impl<T: BufferedReader<C>, C> Reserve<T, C> {
+impl<T: BufferedReader<C>, C: fmt::Debug + Sync + Send> Reserve<T, C> {
     /// Like `new()`, but sets a cookie.
     ///
     /// The cookie can be retrieved using the `cookie_ref` and
@@ -49,14 +47,14 @@ impl<T: BufferedReader<C>, C> Reserve<T, C> {
     pub fn with_cookie(reader: T, reserve: usize, cookie: C)
             -> Reserve<T, C> {
         Reserve {
-            reader: reader,
-            reserve: reserve,
-            cookie: cookie,
+            reader,
+            reserve,
+            cookie,
         }
     }
 }
 
-impl<T: BufferedReader<C>, C> io::Read for Reserve<T, C> {
+impl<T: BufferedReader<C>, C: fmt::Debug + Sync + Send> io::Read for Reserve<T, C> {
     fn read(&mut self, buf: &mut [u8]) -> Result<usize, io::Error> {
         let to_read = {
             let data = self.reader.data(buf.len() + self.reserve)?;
@@ -73,7 +71,7 @@ impl<T: BufferedReader<C>, C> io::Read for Reserve<T, C> {
     }
 }
 
-impl<T: BufferedReader<C>, C> BufferedReader<C> for Reserve<T, C> {
+impl<T: BufferedReader<C>, C: fmt::Debug + Send + Sync> BufferedReader<C> for Reserve<T, C> {
     fn buffer(&self) -> &[u8] {
         let buf = self.reader.buffer();
         if buf.len() > self.reserve {
@@ -173,7 +171,7 @@ mod test {
         // orig: [      | to_read  |          |             ]
         //        \          total           /
         //
-        fn read_chunk<'a, R: BufferedReader<C>, C>(
+        fn read_chunk<'a, R: BufferedReader<C>, C: fmt::Debug + Sync + Send>(
             orig: &[u8], r: &mut R, to_read: usize, cursor: usize, total: usize,
             mode: usize)
         {
